import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_media/nui_camera_util.dart';
import 'package:nui_media/nui_media.dart';

enum SelectionType{
  IMAGE,
  VIDEO
}

class NUISelectMediaDialogUI extends StatefulWidget {
  NUISelectMediaDialogUI({this.title: "Select from", this.titleStyle, this.selectionStyle, this.iconColor, this.galleryIcon, this.cameraIcon, this.recordIcon, this.enableGallery:true, this.enableCamera: true, this.enableRecord: true, this.listener, Key key}): super(key : key);
  final String title;
  final TextStyle titleStyle;
  final TextStyle selectionStyle;
  final Color iconColor;
  final Widget galleryIcon;
  final Widget cameraIcon;
  final Widget recordIcon;
  final bool enableGallery;
  final bool enableCamera;
  final bool enableRecord;
  final MediaSelectionListener listener;

  @override
  _NUISelectMediaDialogUIState createState() => _NUISelectMediaDialogUIState();
}

class _NUISelectMediaDialogUIState extends State<NUISelectMediaDialogUI> {
  void takePhoto() async{
    NUICameraUtil.openCamera(context, enableRecording: true, listener: widget.listener);
  }

  void recordVideo(){

  }

  void chooseFromGallery({SelectionType type: SelectionType.IMAGE}) async{
    if(type == SelectionType.IMAGE) {
      final result = await NUIMediaUtil.chooseImageFromGallery();
      if(widget.listener != null) widget.listener([result]);
    }
    else{
      final result = await NUIMediaUtil.chooseVideoFromGallery();
      if(widget.listener != null) widget.listener([result]);
    }
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> selections = List();

    selections.add(Padding(
      padding: const EdgeInsets.only(left: 15, right: 15, bottom: 5, top: 5),
      child: Text(widget.title ?? "Select from", style: widget.titleStyle ?? TextStyle(fontSize: 18, color: NUIColors.NUITextBlack, fontWeight: FontWeight.bold)),
    ));

    if(widget.enableGallery ?? true){
      selections.add(NUIRippleContainer(
        radius: 5,
        borderRadius: BorderRadius.circular(5),
        child: Padding(
          padding: const EdgeInsets.all(10),
          child: NUIRow(
              children: [
                widget.cameraIcon ?? loadSvg('nui_media_vect_gallery.svg', package: "nui_media", width: 24, color: widget.iconColor ?? NUIColors.NUITextBlack),
                RowText(Text("Choose from gallery", style: widget.selectionStyle ?? TextStyle(fontSize: 16, color: NUIColors.NUITextBlack)))
              ],
              divider: VerticalDivider(width: 10, color: NUIColors.NUITransparent)
          ),
        ),
        onTap: (){
          chooseFromGallery();
        },
      ));
    }
    if(widget.enableCamera ?? true){
      selections.add( NUIRippleContainer(
        radius: 5,
        borderRadius: BorderRadius.circular(5),
        child: Padding(
          padding: const EdgeInsets.all(10),
          child: NUIRow(
              children: [
                widget.cameraIcon ?? loadSvg('nui_media_vect_camera.svg', package: "nui_media", width: 24, color: widget.iconColor ?? NUIColors.NUITextBlack),
                RowText(Text("Take a photo", style: widget.selectionStyle ?? TextStyle(fontSize: 16, color: NUIColors.NUITextBlack)))
              ],
              divider: VerticalDivider(width: 10, color: NUIColors.NUITransparent)
          ),
        ),
        onTap: (){
          takePhoto();
        },
      ));
    }
    if(widget.enableRecord ?? true){
      selections.add(NUIRippleContainer(
        radius: 5,
        borderRadius: BorderRadius.circular(5),
        child: Padding(
          padding: const EdgeInsets.all(10),
          child: NUIRow(
              children: [
                widget.recordIcon ?? loadSvg('nui_media_vect_video.svg', package: "nui_media", width: 24, color: widget.iconColor ?? NUIColors.NUITextBlack),
                RowText(Text("Record a video", style: widget.selectionStyle ?? TextStyle(fontSize: 16, color: NUIColors.NUITextBlack)))
              ],
              divider: VerticalDivider(width: 10, color: NUIColors.NUITransparent)
          ),
        ),
        onTap: (){
          recordVideo();
        },
      ));
    }

    return Container(
      margin: EdgeInsets.all(25),
      padding: EdgeInsets.only(left: 5, right: 5, top: 5, bottom: 10),
      decoration: BoxDecoration(
          color: NUIColors.NUIWhite,
          borderRadius: BorderRadius.circular(15)
      ),
      child: NUIColumn(
        divider: Divider(height: 5, color: NUIColors.NUITransparent),
        mainAxisSize: MainAxisSize.min,
        children: [
          BottomSheetBarIndicatorUI(),
          Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisAlignment: MainAxisAlignment.center,
            mainAxisSize: MainAxisSize.min,
            children: selections
          )
        ],
      ),
    );
  }
}
