import 'dart:ui';
import 'package:better_player/better_player.dart';
import 'package:crop/crop.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:line_awesome_flutter/line_awesome_flutter.dart';
import 'package:nui_animation/nui_animation.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_media/model/nui_media_file.dart';
import 'package:nui_media/nui_media.dart';
import 'package:nui_media/ui/nui_media_cropview.dart';

NUIDialog showMediaPagerPreview(BuildContext context, {@required List<NUIMediaFile> files, int initialPage: 0}){
  final dialog = showNUIDialog(context: context, content: (context){
    return NUIMediaPagerPreview(files: files, initialPage: initialPage, showClose: true);
  }, transition: PageTransition.BLUR_BACKGROUND);
  return dialog;
}

BetterPlayerControlsConfiguration getNUIStyleControlsConfiguration(bool showButtons, bool enableFullScreen){
  return BetterPlayerControlsConfiguration(
    playIcon: LineAwesomeIcons.play,
    pauseIcon: LineAwesomeIcons.pause,
    muteIcon: LineAwesomeIcons.volume_up,
    unMuteIcon: LineAwesomeIcons.volume_mute,
    fullscreenEnableIcon: LineAwesomeIcons.expand,
    fullscreenDisableIcon: LineAwesomeIcons.compress,
    enableSkips: false,
    enableFullscreen: enableFullScreen ?? true,
    controlBarColor: NUIColors.NUIBlack.withOpacity(0.5),
    showControls: showButtons
  );
}

typedef NUIMediaPageSelectedListener = void Function(NUIMediaFile file, int index);

class NUIMediaPagerPreview extends NUIScreen {
  final List<NUIMediaFile> files;
  final int initialPage;
  final bool showClose;
  final NUIMediaPageSelectedListener listener;
  final bool withHeightAnimation;
  final double maxHeightRatio;
  final NUIPageViewController controller;
  final bool showController;
  final bool enableCrop;
  final double cropRatio;
  final bool enableFullScreen;
  final bool blurBackground;
  final bool disableScroll;
  final Color blurBackgroundColor;
  final bool showPageIndicator;
  final NUIMediaCropListener croppedListener;
  final EdgeInsets pageIndicatorPadding;
  final Color pageIndicatorColor;

  NUIMediaPagerPreview({
    Key key,
    @required this.files,
    this.enableFullScreen: true,
    this.croppedListener,
    this.cropRatio: 1.0,
    this.enableCrop: false,
    this.initialPage,
    this.controller,
    this.showPageIndicator: true,
    this.blurBackground: false,
    this.blurBackgroundColor,
    this.showController: true,
    double maxWidth,
    this.showClose: false,
    this.listener,
    this.pageIndicatorColor,
    this.withHeightAnimation: false,
    this.disableScroll: false,
    bool darkStatusBarIcon: false,
    this.pageIndicatorPadding,
    this.maxHeightRatio: 1.2}) : super(
      content : (context) => NUIMediaPagerView(
        files: files,
        initialPage: initialPage,
        croppedListener: croppedListener,
        cropRatio: cropRatio,
        maxWidth: maxWidth,
        enableCrop: enableCrop,
        enableFullScreen: enableFullScreen,
        showClose: showClose,
        showController:showController,
        controller: controller,
        listener: listener,
        showPageIndicator: showPageIndicator,
        withHeightAnimation: withHeightAnimation,
        maxHeightRatio: maxHeightRatio,
        disableScroll: disableScroll,
        pageIndicatorPadding: pageIndicatorPadding,
        blurBackground: blurBackground,
        pageIndicatorBackgroundColor: pageIndicatorColor,
        blurBackgroundColor: blurBackgroundColor,
      ), screenCode: "NUIMediaPagerPreviewUI", key: key, overlayStyle: NUIOverlayStyle(darkStatusBarIcon: darkStatusBarIcon ?? false));
}

typedef NUIMediaCropListener = void Function(NUIMediaFile croppedFile);

class NUIMediaPagerView extends StatefulWidget {
  NUIMediaPagerView({
    @required this.files,
    this.initialPage : 0,
    this.controller,
    this.croppedListener,
    this.maxWidth,
    this.cropRatio: 1.0,
    this.enableCrop: false,
    this.enableFullScreen: true,
    this.showController: true,
    this.showClose: false,
    this.listener,
    this.blurBackground,
    this.blurBackgroundColor,
    this.withHeightAnimation: false,
    this.showPageIndicator: true,
    this.pageIndicatorBackgroundColor,
    this.pageIndicatorPadding,
    this.disableScroll: false,
    this.maxHeightRatio});

  final List<NUIMediaFile> files;
  final int initialPage;
  final bool showClose;
  final NUIMediaPageSelectedListener listener;
  final bool withHeightAnimation;
  final bool enableFullScreen;
  final NUIPageViewController controller;
  final double maxHeightRatio;
  final bool enableCrop;
  final double cropRatio;
  final NUIMediaCropListener croppedListener;
  final bool showController;
  final double maxWidth;
  final bool disableScroll;
  final Color pageIndicatorBackgroundColor;
  final bool blurBackground;
  final Color blurBackgroundColor;
  final bool showPageIndicator;
  final EdgeInsets pageIndicatorPadding;

  @override
  _NUIMediaPagerViewState createState() => _NUIMediaPagerViewState();
}

class _NUIMediaPagerViewState extends State<NUIMediaPagerView> with TickerProviderStateMixin{
  NUIListViewBloc<NUIMediaPagerItem, NUIMediaPagerItem> bloc;
  int _currentPage;
  double maxWidth;

  @override
  void initState(){
    super.initState();
    _currentPage = widget.initialPage ?? 0;
    widget.files.forEach((e) {
      e.maxHeightToWidthRatio = widget.maxHeightRatio ?? 1.2;
    });
    bloc = NUIListViewBloc<NUIMediaPagerItem, NUIMediaPagerItem>(callbackMethod: (List<NUIMediaPagerItem> current, bool isFirst, bool append, NUIMediaPagerItem cursor) async{
      final images = List<NUIMediaPagerItem>();
      widget.files.forEach((e) {
        images.add(e.thumbData == null  && e.videoPath.isNullOrEmpty() ? NUIMediaPagerImage(e, showController: widget.showController) : NUIMediaPagerVideo(e, showController: widget.showController));
      });
      final result = ListResult<NUIMediaPagerItem, NUIMediaPagerItem>(data: images, success: true, message: "Parse medias successful of size : ${images.length}", cursor: null);
      return result;
    });
  }

  Widget _getPageIndicator(){
    return Visibility(
      visible: widget.files.length > 1 && widget.showPageIndicator == true,
      child: NUIClipView(
        borderRadius: BorderRadius.circular(15),
        child: BackdropFilter(
          filter: ImageFilter.blur(sigmaX: 4, sigmaY: 4),
          child: Container(
              height: 30,
              padding: EdgeInsets.only(left: 7, right: 7),
              alignment: Alignment.center,
              decoration: NUIBoxDecoration(
                borderRadius: BorderRadius.circular(15),
                gradient: NUILinearGradient(
                  colors: [
                    widget.pageIndicatorBackgroundColor ?? NUIColors.NUIBlack.withOpacity(0.2),
                    widget.pageIndicatorBackgroundColor ?? NUIColors.NUIBlack.withOpacity(0.1),
                  ],
                  orientation: GradientOrientation.TL_BR
                ),
              ),
              child: Text("${_currentPage + 1}/${widget.files.length}", style: TextStyle(color: NUIColors.NUIWhite, fontSize: 12, fontWeight: FontWeight.bold, decoration: TextDecoration.none, shadows: [Shadow(color: Colors.black.withOpacity(0.1), blurRadius: 2, offset: Offset(1,1))]))
            ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    maxWidth = widget.maxWidth ?? screenWidth(context);
    final statusBarHeight = NUIDisplayUtil.statusBarHeight(context);
    final content = Stack(
      children: [
        NUIPageView<NUIMediaPagerItem, NUIMediaPagerItem>(
            bloc: bloc,
            viewportFraction: 1.0,
            initialPosition: _currentPage,
            withItemCount: false,
            enablePagination: false,
            hardLoad: true,
            disableScroll: widget.disableScroll ?? false,
            configMap: {
              "enableFullScreen": widget.enableFullScreen,
              "enableCrop": widget.enableCrop,
              "cropRatio": widget.cropRatio,
              "cropListener": widget.croppedListener,
              "blurBackground": widget.blurBackground,
              "blurBackgroundColor": widget.blurBackgroundColor
            },
            controller: widget.controller,
            repo: NUIMediaListItemRepo(),
            initialList: List<NUIMediaPagerItem>(),
            clickListener: (data, position, action){

            },
            pageChangeListener: (page){
              if(widget.listener != null) widget.listener(null, page);
              setState(() {
                _currentPage = page;
              });
            },
        ),
        Visibility(
          visible: widget.showClose == true,
          child: Align(
            alignment: Alignment.topCenter,
            child: Container(
              height: NUIDisplayUtil.statusBarHeight(context) + 20,
              width: maxWidth,
              decoration: BoxDecoration(
                gradient: NUILinearGradient(
                  colors: [
                    NUIColors.NUIShadowBlack.withOpacity(0.5),
                    NUIColors.NUIShadowBlack.withOpacity(0.15),
                    NUIColors.NUITransparent
                  ],
                  orientation: GradientOrientation.T_B
                )
              ),
            ),
          ),
        ),
        Visibility(
          visible: widget.showClose == true,
          child: Padding(
            padding: EdgeInsets.only(top: statusBarHeight, left: 10, right: 10),
            child: Align(
              alignment: Alignment.topLeft,
              child: NUIShadowView(
                radius: 30,
                blurRadius: 15,
                opacity: 0.1,
                child: Container(
                  margin: EdgeInsets.only(left: 5),
                  child: NUIBouncingAnimation(
                    child: Padding(
                      padding: EdgeInsets.all(5),
                      child: Icon(LineAwesomeIcons.angle_left, color: NUIColors.NUIWhite, size: 24)
                    ),
                    onTap: (){
                      NUINavigator.pop(context);
                    },
                  ),
                ),
              ),
            ),
          ),
        ),
        Positioned(
          top: widget.showClose == true ? statusBarHeight + 5 : 5,
          right: 5,
          child: Padding(
            padding: widget.pageIndicatorPadding ?? EdgeInsets.all(0),
            child: _getPageIndicator(),
          )
        )
      ],
    );

    if(widget.withHeightAnimation == true){
      final calculatedHeight = NUIMediaUtil.heightByMediaRatio(context, widget.files[_currentPage], maxWidth);
      return AnimatedSize(
        duration: Duration(milliseconds: 250),
        alignment: Alignment.center,
        child: Container(
          height: calculatedHeight,
          width: double.infinity,
          child: content,
        ),
        vsync: this
      );
    }
    return Container(child: content);
  }
}


class NUIImagePreviewContent extends NUIListStatefulWidget {
  NUIImagePreviewContent(this.file, {NUIListActionController handler}): super(controller: handler);
  final NUIMediaPagerImage file;

  @override
  _NUIImagePreviewContentState createState() => _NUIImagePreviewContentState();
}

class _NUIImagePreviewContentState extends NUIListState<NUIImagePreviewContent> {
  final controller = CropController(aspectRatio: 0.7);
  double _rotation = 0;
  BoxShape shape = BoxShape.rectangle;

  void _cropImage() async {
    final pixelRatio = MediaQuery.of(context).devicePixelRatio;
    final cropped = await controller.crop(pixelRatio: pixelRatio);
  }

  @override
  Widget build(BuildContext context) {
    final isLocalImage = widget.file.file.isLocal == true;
    final enableCrop = getConfigData("enableCrop") == true;
    final cropRatio = getConfigData("cropRatio");
    final blurBackgroundColor = getConfigData("blurBackgroundColor");
    final blurBackground = getConfigData("blurBackground") == true;
    NUIMediaCropListener cropListener = getConfigData("cropListener");

    return Container(
      child: Container(
        child: Stack(
          children: [
            blurBackground == true ? NUIClipView(
              borderRadius: BorderRadius.circular(0),
              child: Container(
                width: double.infinity,
                height: double.infinity,
                child: ImageFiltered(
                  imageFilter: ImageFilter.blur(sigmaX: 10, sigmaY: 10),
                  child: isLocalImage
                      ? Image.file(widget.file.file.getFile())
                      : loadImage(widget.file.file.path, fit: BoxFit.cover),
                ),
                foregroundDecoration: NUIBoxDecoration(
                  color: blurBackgroundColor ?? Colors.transparent
                ),
              ),
            ) : SizedBox.shrink(),
            Center(
              child: NUIZoomOverlay(
                twoTouchOnly: true,
                child: Hero(
                  tag: widget.file.file.path,
                  child: isLocalImage
                      ? Image.file(widget.file.file.getFile())
                      : loadImage(widget.file.file.path, fit: BoxFit.cover),
                )
              ),
            ),
            Visibility(
              visible: enableCrop,
              child: Positioned(
                bottom: 10,
                right: 15,
                child: NUIBouncingAnimation(
                  scaleFactor: 0.5,
                  onTap: (){
                    //Crop the image maybe
                    showNUIMediaCropping(context, mediaFile: widget.file.file, ratio: cropRatio ?? 1.0, saveAsFile: true).then((value){
                      if(value != null){
                        if(cropListener != null) cropListener(value);
                      }
                    });
                  },
                  child: NUIClipView(
                    borderRadius: BorderRadius.circular(18),
                    child: BackdropFilter(
                      filter: ImageFilter.blur(sigmaX: 5.0, sigmaY: 5.0),
                      child: Container(
                          height: 36,
                          width: 36,
                          alignment: Alignment.center,
                          decoration: NUIOutlineDecoration(
                              outlineColor: NUIColors.NUIWhite.withOpacity(0.6),
                              gradient: NUILinearGradient(
                                  colors: [
                                    NUIColors.NUIBlack.withOpacity(0.2),
                                    NUIColors.NUIBlack.withOpacity(0.1),
                                  ],
                                  orientation: GradientOrientation.TL_BR
                              ),
                              thickness: 1.5,
                              borderRadius: BorderRadius.circular(18)
                          ),
                          child: Icon(LineAwesomeIcons.crop, size: 20, color: NUIColors.NUIWhite,)
                      ),
                    ),
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }
}

class NUIVideoPreviewContent extends NUIListStatefulWidget {
  NUIVideoPreviewContent(this.file, {NUIListActionController handler}): super(controller: handler);
  final NUIMediaPagerVideo file;
  @override
  _NUIVideoPreviewContentState createState() => _NUIVideoPreviewContentState();
}

class _NUIVideoPreviewContentState extends NUIListState<NUIVideoPreviewContent> {
  @override
  Widget build(BuildContext context) {
    final isLocalImage = widget.file.file.isLocal == true;
    final blurBackground = getConfigData("blurBackground") == true;
    final blurBackgroundColor = getConfigData("blurBackgroundColor");
    final enableFullScreen = getConfigData("enableFullScreen") == true;
    return Container(
      child: Stack(
        children: [
          blurBackground == true ? NUIClipView(
            borderRadius: BorderRadius.circular(0),
            child: Container(
              width: double.infinity,
              height: double.infinity,
              child: ImageFiltered(
                imageFilter: ImageFilter.blur(sigmaX: 10, sigmaY: 10),
                child: isLocalImage
                    ? Image.file(widget.file.file.getFile())
                    : loadImage(widget.file.file.path, fit: BoxFit.cover),
              ),
              foregroundDecoration: NUIBoxDecoration(
                color: blurBackgroundColor ?? Colors.transparent
              ),
            ),
          ) : SizedBox.shrink(),
          Center(
            child: NUIVideoListViewPlayer(
                mediaFile: widget.file.file,
                autoPlay: true,
                playFraction: 0.8,
                boxFit: BoxFit.cover,
                enableFullScreen: enableFullScreen ?? true,
                aspectRatio: widget.file.file.widthToHeightRatio(),
                showButtons: widget.file.showController == true,
            ),
          ),
        ],
      ),
    );
  }
}
